/*
    Group and Word Classes (code) : part of CalcH1Symplectic

    Copyright 2009 Andrew Putman.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include <vector>
#include <iostream>
#include <time.h>
#include "Matrix.h"
#include "Group.h"

void Word::RightMult(CounterType gen, CounterType sign)
{
	gens.push_back(gen);
	signs.push_back(sign);
}

Word Word::operator*(Word rhs)
{
        Word ret;
        CounterType i;

        ret = *this;

        for (i=0;i<rhs.Length();i++)
                ret.RightMult(rhs.Letter(i), rhs.Sign(i));

        return(ret);
}

Word Inverse(Word toInvert)
{
	Word ret;
	CounterType i;

	for (i=0;i<toInvert.Length();i++)
		ret.RightMult(toInvert.Letter(toInvert.Length() - i - 1), (0-1)*toInvert.Sign(toInvert.Length() - i - 1));
	return(ret);
}

Word Pow(Word in, int exponent)
{
	Word ret;
	CounterType i;
	
	if (exponent < 0)
	{
		in = Inverse(in);
		exponent = exponent * (0-1);
	}

	for (i=0;i<(CounterType)exponent;i++)
		ret = ret * in;

	return(ret);
}

CounterType Pow(CounterType x, CounterType y)
{
	CounterType ret = 1;

	for (CounterType i=0;i<y;i++)
		ret = ret * x;

	return(ret);
}

Word Gen(CounterType num)
{
	Word ret;
	ret.RightMult(num,1);
	return(ret);
}
